@@ -33,14 +33,16 @@ size_t Data::getVariableID(const std::string& variable_name) const {
   return (std::distance(variable_names.cbegin(), it));
 }
 
+// #nocov start (cannot be tested anymore because GenABEL not on CRAN)
 void Data::addSnpData(unsigned char* snp_data, size_t num_cols_snp) {
   num_cols = num_cols_no_snp + num_cols_snp;
   num_rows_rounded = roundToNextMultiple(num_rows, 4);
   this->snp_data = snp_data;
 }
+// #nocov end
 
 // #nocov start
-bool Data::loadFromFile(std::string filename) {
+bool Data::loadFromFile(std::string filename, std::vector<std::string>& dependent_variable_names) {
 
   bool result;
 
@@ -61,17 +63,17 @@ bool Data::loadFromFile(std::string filename) {
   input_file.close();
   input_file.open(filename);
 
-  // Check if comma, semicolon or whitespace seperated
+  // Check if comma, semicolon or whitespace separated
   std::string header_line;
   getline(input_file, header_line);
 
-  // Find out if comma, semicolon or whitespace seperated and call appropriate method
-  if (header_line.find(",") != std::string::npos) {
-    result = loadFromFileOther(input_file, header_line, ',');
-  } else if (header_line.find(";") != std::string::npos) {
-    result = loadFromFileOther(input_file, header_line, ';');
+  // Find out if comma, semicolon or whitespace separated and call appropriate method
+  if (header_line.find(',') != std::string::npos) {
+    result = loadFromFileOther(input_file, header_line, dependent_variable_names, ',');
+  } else if (header_line.find(';') != std::string::npos) {
+    result = loadFromFileOther(input_file, header_line, dependent_variable_names, ';');
   } else {
-    result = loadFromFileWhitespace(input_file, header_line);
+    result = loadFromFileWhitespace(input_file, header_line, dependent_variable_names);
   }
 
   externalData = false;
@@ -79,19 +81,36 @@ bool Data::loadFromFile(std::string filename) {
   return result;
 }
 
-bool Data::loadFromFileWhitespace(std::ifstream& input_file, std::string header_line) {
+bool Data::loadFromFileWhitespace(std::ifstream& input_file, std::string header_line,
+    std::vector<std::string>& dependent_variable_names) {
+
+  size_t num_dependent_variables = dependent_variable_names.size();
+  std::vector<size_t> dependent_varIDs;
+  dependent_varIDs.resize(num_dependent_variables);
 
   // Read header
   std::string header_token;
   std::stringstream header_line_stream(header_line);
+  size_t col = 0;
   while (header_line_stream >> header_token) {
-    variable_names.push_back(header_token);
+    bool is_dependent_var = false;
+    for (size_t i = 0; i < dependent_variable_names.size(); ++i) {
+      if (header_token == dependent_variable_names[i]) {
+        dependent_varIDs[i] = col;
+        is_dependent_var = true;
+      }
+    }
+    if (!is_dependent_var) {
+      variable_names.push_back(header_token);
+    }
+    ++col;
   }
+
   num_cols = variable_names.size();
   num_cols_no_snp = num_cols;
 
   // Read body
-  reserveMemory();
+  reserveMemory(num_dependent_variables);
   bool error = false;
   std::string line;
   size_t row = 0;
@@ -100,13 +119,26 @@ bool Data::loadFromFileWhitespace(std::ifstream& input_file, std::string header_
     std::stringstream line_stream(line);
     size_t column = 0;
     while (readFromStream(line_stream, token)) {
-      set(column, row, token, error);
+      size_t column_x = column;
+      bool is_dependent_var = false;
+      for (size_t i = 0; i < dependent_varIDs.size(); ++i) {
+        if (column == dependent_varIDs[i]) {
+          set_y(i, row, token, error);
+          is_dependent_var = true;
+          break;
+        } else if (column > dependent_varIDs[i]) {
+          --column_x;
+        }
+      }
+      if (!is_dependent_var) {
+        set_x(column_x, row, token, error);
+      }
       ++column;
     }
-    if (column > num_cols) {
+    if (column > (num_cols + num_dependent_variables)) {
       throw std::runtime_error(
           std::string("Could not open input file. Too many columns in row ") + std::to_string(row) + std::string("."));
-    } else if (column < num_cols) {
+    } else if (column < (num_cols + num_dependent_variables)) {
       throw std::runtime_error(
           std::string("Could not open input file. Too few columns in row ") + std::to_string(row)
               + std::string(". Are all values numeric?"));
@@ -117,19 +149,36 @@ bool Data::loadFromFileWhitespace(std::ifstream& input_file, std::string header_
   return error;
 }
 
-bool Data::loadFromFileOther(std::ifstream& input_file, std::string header_line, char seperator) {
+bool Data::loadFromFileOther(std::ifstream& input_file, std::string header_line,
+    std::vector<std::string>& dependent_variable_names, char separator) {
+
+  size_t num_dependent_variables = dependent_variable_names.size();
+  std::vector<size_t> dependent_varIDs;
+  dependent_varIDs.resize(num_dependent_variables);
 
   // Read header
   std::string header_token;
   std::stringstream header_line_stream(header_line);
-  while (getline(header_line_stream, header_token, seperator)) {
-    variable_names.push_back(header_token);
+  size_t col = 0;
+  while (getline(header_line_stream, header_token, separator)) {
+    bool is_dependent_var = false;
+    for (size_t i = 0; i < dependent_variable_names.size(); ++i) {
+      if (header_token == dependent_variable_names[i]) {
+        dependent_varIDs[i] = col;
+        is_dependent_var = true;
+      }
+    }
+    if (!is_dependent_var) {
+      variable_names.push_back(header_token);
+    }
+    ++col;
   }
+
   num_cols = variable_names.size();
   num_cols_no_snp = num_cols;
 
   // Read body
-  reserveMemory();
+  reserveMemory(num_dependent_variables);
   bool error = false;
   std::string line;
   size_t row = 0;
@@ -138,10 +187,24 @@ bool Data::loadFromFileOther(std::ifstream& input_file, std::string header_line,
     double token;
     std::stringstream line_stream(line);
     size_t column = 0;
-    while (getline(line_stream, token_string, seperator)) {
+    while (getline(line_stream, token_string, separator)) {
       std::stringstream token_stream(token_string);
       readFromStream(token_stream, token);
-      set(column, row, token, error);
+
+      size_t column_x = column;
+      bool is_dependent_var = false;
+      for (size_t i = 0; i < dependent_varIDs.size(); ++i) {
+        if (column == dependent_varIDs[i]) {
+          set_y(i, row, token, error);
+          is_dependent_var = true;
+          break;
+        } else if (column > dependent_varIDs[i]) {
+          --column_x;
+        }
+      }
+      if (!is_dependent_var) {
+        set_x(column_x, row, token, error);
+      }
       ++column;
     }
     ++row;
@@ -151,14 +214,15 @@ bool Data::loadFromFileOther(std::ifstream& input_file, std::string header_line,
 }
 // #nocov end
 
-void Data::getAllValues(std::vector<double>& all_values, std::vector<size_t>& sampleIDs, size_t varID, size_t start, size_t end) const {
+void Data::getAllValues(std::vector<double>& all_values, std::vector<size_t>& sampleIDs, size_t varID, size_t start,
+    size_t end) const {
 
   // All values for varID (no duplicates) for given sampleIDs
   if (getUnpermutedVarID(varID) < num_cols_no_snp) {
-
-    all_values.reserve(end-start);
+    
+    all_values.reserve(end - start);
     for (size_t pos = start; pos < end; ++pos) {
-      all_values.push_back(get(sampleIDs[pos], varID));
+      all_values.push_back(get_x(sampleIDs[pos], varID));
     }
     std::sort(all_values.begin(), all_values.end());
     all_values.erase(std::unique(all_values.begin(), all_values.end()), all_values.end());
@@ -168,13 +232,14 @@ void Data::getAllValues(std::vector<double>& all_values, std::vector<size_t>& sa
   }
 }
 
-void Data::getMinMaxValues(double& min, double&max, std::vector<size_t>& sampleIDs, size_t varID, size_t start, size_t end) const {
+void Data::getMinMaxValues(double& min, double&max, std::vector<size_t>& sampleIDs, size_t varID, size_t start,
+    size_t end) const {
   if (sampleIDs.size() > 0) {
-    min = get(sampleIDs[0], varID);
+    min = get_x(sampleIDs[start], varID);
     max = min;
   }
   for (size_t pos = start; pos < end; ++pos) {
-    double value = get(sampleIDs[pos], varID);
+    double value = get_x(sampleIDs[pos], varID);
     if (value < min) {
       min = value;
     }
@@ -195,14 +260,15 @@ void Data::sort() {
     // Get all unique values
     std::vector<double> unique_values(num_rows);
     for (size_t row = 0; row < num_rows; ++row) {
-      unique_values[row] = get(row, col);
+      unique_values[row] = get_x(row, col);
     }
     std::sort(unique_values.begin(), unique_values.end());
     unique_values.erase(unique(unique_values.begin(), unique_values.end()), unique_values.end());
 
     // Get index of unique value
     for (size_t row = 0; row < num_rows; ++row) {
-      size_t idx = std::lower_bound(unique_values.begin(), unique_values.end(), get(row, col)) - unique_values.begin();
+      size_t idx = std::lower_bound(unique_values.begin(), unique_values.end(), get_x(row, col))
+          - unique_values.begin();
       index_data[col * num_rows + row] = idx;
     }
 
@@ -215,13 +281,13 @@ void Data::sort() {
 }
 
 // TODO: Implement ordering for multiclass and survival
-void Data::orderSnpLevels(std::string dependent_variable_name, bool corrected_importance) {
+// #nocov start (cannot be tested anymore because GenABEL not on CRAN)
+void Data::orderSnpLevels(bool corrected_importance) {
   // Stop if now SNP data
   if (snp_data == 0) {
     return;
   }
 
-  size_t dependent_varID = getVariableID(dependent_variable_name);
   size_t num_snps;
   if (corrected_importance) {
     num_snps = 2 * (num_cols - num_cols_no_snp);
@@ -256,7 +322,7 @@ void Data::orderSnpLevels(std::string dependent_variable_name, bool corrected_im
         value = 0;
       }
 
-      means[value] += get(row, dependent_varID);
+      means[value] += get_y(row, 0);
       ++counts[value];
     }
 
@@ -270,6 +336,7 @@ void Data::orderSnpLevels(std::string dependent_variable_name, bool corrected_im
 
   order_snps = true;
 }
+// #nocov end
 
 } // namespace ranger
 
