@@ -18,13 +18,12 @@
 
 namespace ranger {
 
-void ForestProbability::loadForest(size_t dependent_varID, size_t num_trees,
+void ForestProbability::loadForest(size_t num_trees,
     std::vector<std::vector<std::vector<size_t>> >& forest_child_nodeIDs,
     std::vector<std::vector<size_t>>& forest_split_varIDs, std::vector<std::vector<double>>& forest_split_values,
     std::vector<double>& class_values, std::vector<std::vector<std::vector<double>>>& forest_terminal_class_counts,
     std::vector<bool>& is_ordered_variable) {
 
-  this->dependent_varID = dependent_varID;
   this->num_trees = num_trees;
   this->class_values = class_values;
   data->setIsOrderedVariable(is_ordered_variable);
@@ -33,7 +32,7 @@ void ForestProbability::loadForest(size_t dependent_varID, size_t num_trees,
   trees.reserve(num_trees);
   for (size_t i = 0; i < num_trees; ++i) {
     trees.push_back(
-        make_unique<TreeProbability>(forest_child_nodeIDs[i], forest_split_varIDs[i], forest_split_values[i],
+        std::make_unique<TreeProbability>(forest_child_nodeIDs[i], forest_split_varIDs[i], forest_split_values[i],
             &this->class_values, &response_classIDs, forest_terminal_class_counts[i]));
   }
 
@@ -51,11 +50,11 @@ std::vector<std::vector<std::vector<double>>> ForestProbability::getTerminalClas
   return result;
 }
 
-void ForestProbability::initInternal(std::string status_variable_name) {
+void ForestProbability::initInternal() {
 
   // If mtry not set, use floored square root of number of independent variables.
   if (mtry == 0) {
-    unsigned long temp = sqrt((double) (num_variables - 1));
+    unsigned long temp = sqrt((double) num_independent_variables);
     mtry = std::max((unsigned long) 1, temp);
   }
 
@@ -64,10 +63,15 @@ void ForestProbability::initInternal(std::string status_variable_name) {
     min_node_size = DEFAULT_MIN_NODE_SIZE_PROBABILITY;
   }
 
+  // Set minimal bucket size
+  if (min_bucket == 0) {
+    min_bucket = DEFAULT_MIN_BUCKET;
+  }
+
   // Create class_values and response_classIDs
   if (!prediction_mode) {
     for (size_t i = 0; i < num_samples; ++i) {
-      double value = data->get(i, dependent_varID);
+      double value = data->get_y(i, 0);
 
       // If classID is already in class_values, use ID. Else create a new one.
       uint classID = find(class_values.begin(), class_values.end(), value) - class_values.begin();
@@ -76,6 +80,10 @@ void ForestProbability::initInternal(std::string status_variable_name) {
       }
       response_classIDs.push_back(classID);
     }
+
+    if (splitrule == HELLINGER && class_values.size() != 2) {
+      throw std::runtime_error("Hellinger splitrule only implemented for binary classification.");
+    }
   }
 
   // Create sampleIDs_per_class if required
@@ -103,7 +111,7 @@ void ForestProbability::growInternal() {
   trees.reserve(num_trees);
   for (size_t i = 0; i < num_trees; ++i) {
     trees.push_back(
-        make_unique<TreeProbability>(&class_values, &response_classIDs, &sampleIDs_per_class, &class_weights));
+        std::make_unique<TreeProbability>(&class_values, &response_classIDs, &sampleIDs_per_class, &class_weights));
   }
 }
 
@@ -151,7 +159,7 @@ void ForestProbability::predictInternal(size_t sample_idx) {
 
 void ForestProbability::computePredictionErrorInternal() {
 
-// For each sample sum over trees where sample is OOB
+  // For each sample sum over trees where sample is OOB
   std::vector<size_t> samples_oob_count;
   samples_oob_count.resize(num_samples, 0);
   predictions = std::vector<std::vector<std::vector<double>>>(1,
@@ -169,8 +177,9 @@ void ForestProbability::computePredictionErrorInternal() {
     }
   }
 
-// MSE with predicted probability and true data
+  // MSE with predicted probability and true data
   size_t num_predictions = 0;
+  overall_prediction_error = 0;
   for (size_t i = 0; i < predictions[0].size(); ++i) {
     if (samples_oob_count[i] > 0) {
       ++num_predictions;
@@ -199,7 +208,7 @@ void ForestProbability::writeOutputInternal() {
 
 void ForestProbability::writeConfusionFile() {
 
-// Open confusion file for writing
+  // Open confusion file for writing
   std::string filename = output_prefix + ".confusion";
   std::ofstream outfile;
   outfile.open(filename, std::ios::out);
@@ -207,7 +216,7 @@ void ForestProbability::writeConfusionFile() {
     throw std::runtime_error("Could not write to confusion file: " + filename + ".");
   }
 
-// Write confusion to file
+  // Write confusion to file
   outfile << "Overall OOB prediction error (MSE): " << overall_prediction_error << std::endl;
 
   outfile.close();
@@ -260,31 +269,31 @@ void ForestProbability::writePredictionFile() {
 
 void ForestProbability::saveToFileInternal(std::ofstream& outfile) {
 
-// Write num_variables
-  outfile.write((char*) &num_variables, sizeof(num_variables));
+  // Write num_variables
+  outfile.write((char*) &num_independent_variables, sizeof(num_independent_variables));
 
-// Write treetype
+  // Write treetype
   TreeType treetype = TREE_PROBABILITY;
   outfile.write((char*) &treetype, sizeof(treetype));
 
-// Write class_values
+  // Write class_values
   saveVector1D(class_values, outfile);
 }
 
 void ForestProbability::loadFromFileInternal(std::ifstream& infile) {
 
-// Read number of variables
+  // Read number of variables
   size_t num_variables_saved;
   infile.read((char*) &num_variables_saved, sizeof(num_variables_saved));
 
-// Read treetype
+  // Read treetype
   TreeType treetype;
   infile.read((char*) &treetype, sizeof(treetype));
   if (treetype != TREE_PROBABILITY) {
     throw std::runtime_error("Wrong treetype. Loaded file is not a probability estimation forest.");
   }
 
-// Read class_values
+  // Read class_values
   readVector1D(class_values, infile);
 
   for (size_t i = 0; i < num_trees; ++i) {
@@ -310,18 +319,14 @@ void ForestProbability::loadFromFileInternal(std::ifstream& infile) {
       terminal_class_counts[terminal_nodes[j]] = terminal_class_counts_vector[j];
     }
 
-    // If dependent variable not in test data, change variable IDs accordingly
-    if (num_variables_saved > num_variables) {
-      for (auto& varID : split_varIDs) {
-        if (varID >= dependent_varID) {
-          --varID;
-        }
-      }
+    // If dependent variable not in test data, throw error
+    if (num_variables_saved != num_independent_variables) {
+      throw std::runtime_error("Number of independent variables in data does not match with the loaded forest.");
     }
 
     // Create tree
     trees.push_back(
-        make_unique<TreeProbability>(child_nodeIDs, split_varIDs, split_values, &class_values, &response_classIDs,
+        std::make_unique<TreeProbability>(child_nodeIDs, split_varIDs, split_values, &class_values, &response_classIDs,
             terminal_class_counts));
   }
 }
