@@ -10,9 +10,9 @@ rg.class <- ranger(Species ~ ., data = iris)
 rg.mat   <- ranger(dependent.variable.name = "Species", data = dat, classification = TRUE)
 
 ## Basic tests (for all random forests equal)
-test_that("classification result is of class ranger with 14 elements", {
+test_that("classification result is of class ranger with 15 elements", {
   expect_is(rg.class, "ranger")
-  expect_equal(length(rg.class), 14)
+  expect_equal(length(rg.class), 15)
 })
 
 test_that("classification prediction returns factor", {
@@ -192,5 +192,22 @@ test_that("No error if unused factor levels in outcome", {
   expect_warning(rf <- ranger(Species ~ ., iris[1:100, ], num.trees = 5),
                  "^Dropped unused factor level\\(s\\) in dependent variable\\: virginica\\.")
   pred <- predict(rf, iris)
-  expect_equal(levels(pred$predictions), levels(droplevels(iris[1:100, "Species"])))
+  expect_equal(levels(pred$predictions), levels(iris$Species))
+})
+
+test_that("Predictions with unused factor levels are not NA", {
+  expect_warning(rf <- ranger(Species ~ ., iris[51:150, ], num.trees = 5),
+                 "^Dropped unused factor level\\(s\\) in dependent variable\\: setosa\\.")
+  pred <- predict(rf, iris)
+  expect_equal(sum(is.na(pred$predictions)), 0)
+})
+
+test_that("classification with logical returns factor", {
+  rf <- ranger(Species=="setosa" ~ ., data = iris, num.trees = 5)
+  expect_is(rf$predictions, "numeric")
+  expect_null(dim(rf$predictions))
+  
+  pred <- predict(rf, iris)
+  expect_is(pred$predictions, "numeric")
+  expect_null(dim(pred$predictions))
 })
