@@ -4,7 +4,7 @@ library(ranger)
 context("ranger_reg")
 
 ## Initialize the random forest for regression
-rg.reg <- ranger(Sepal.Length ~ ., data = iris, verbose = FALSE, write.forest = TRUE)
+rg.reg <- ranger(Sepal.Length ~ ., data = iris)
 
 ## Basic tests (for all random forests equal)
 test_that("regression result is of class ranger with 15 elements", {
